/**
 * \file mgsLib.h
 * \author German Fabregat
 * \date February 2016
 * \brief Header file for the MGS Library
 *
 */

#ifndef _MGSLIB_H
#define _MGSLIB_H

// Public data types
/*
 * Structure to retrieve information about the mgs library
 * using the mgsVersionData() function.
 */
struct mgsInfo{
	int vMajor; // Major version number.
	int vMinor; // Minor version number.
	int	maxScr; // Maximum number of screens.
	int freeScr; // Current number of free screens to be used.
    int maxImg; // Maximum number of images.
	int freeImg; // Current number of free images to be used.
	int frmTime; // Current frame duration in ms.
};

// General screen management
int mgsCreateScreen(int width, int height, char *title, int bckColor, char *bckFile);
int mgsFullScreen(int bckColor, char *bckFile);
int mgsDeleteScreen(int scrNumber);
int mgsMoveScreen(int scr, int x, int y);
int mgsActivateScreen(int scr);
int mgsUpdateScreen(int scrNumber);
int mgsClearScreen(int scrNumber);
int mgsSetBackground(int scrNumber);
int mgsGetScreenInfoAdvanced(int scrNumber, int *posAndSize, char *handle);
// Point and line drawing and testing
int mgsPoint(int scrNumber, int x, int y, int color);
int mgsGetColor(int scr, int x, int y);
int mgsLine(int scrNumber, int xStart, int yStart, int xEnd, int yEnd, int width, int color);
int mgsEArc(int scrNumber, int centerX, int centerY, int radiusH, int radiusV, int startingAngle, int endingAngle, int width, int color);
int mgsRotatedEArc(int scrNumber, int centerX, int centerY, int radiusH, int radiusV, int startingAngle, int endingAngle, int rotationAngle, int width, int color);
// Outlined figures
int mgsDrawRectangle(int scrNumer, int left, int top, int right, int bottom, int width, int color);
int mgsDrawEllipse(int scrNumber, int centerX, int centerY, int radiusH, int radiusV, int width, int color);
int mgsDrawRegPolygonCR(int scrNumber, int sides, int centerX, int centerY, int radius, int width, int color);
int mgsDrawRegPolygonVC(int scrNumber, int sides, int vertexX, int vertexY, int centerX, int centerY, int width, int color);
int mgsDrawPolygon(int scrNumber, int sides, int *vertices, int width, int color);
int mgsDrawStarCR(int scrNumber, int nsides, int centerX, int centerY, int externalRadius, int internalRadius, int width, int color);
int mgsDrawStarVC(int scrNumber, int sides, int vertexX, int vertexY, int centerX, int centerY, int internalRadius, int width, int color);
int mgsDrawESector(int scrNumber, int centerX, int centerY, int radiusH, int radiusV, int startingAngle, int endingAngle, int width, int color);
int mgsDrawESegment(int scrNumber, int centerX, int centerY, int radiusH, int radiusV, int startingAngle, int endingAngle, int width, int color);
// Rotated version
int mgsDrawRotatedRectangle(int scrNumer, int left, int top, int right, int bottom, int angle, int width, int color);
int mgsDrawRotatedEllipse(int scrNumber, int centerX, int centerY, int radiusH, int radiusV, int angle, int width, int color);
int mgsDrawRotatedRegPolygonCR(int scrNumber, int sides, int centerX, int centerY, int radius, int angle, int width, int color);
int mgsDrawRotatedPolygon(int scrNumber, int sides, int *vertices, int centerX, int centerY, int angle, int width, int color);
int mgsDrawRotatedStarCR(int scrNumber, int nsides, int centerX, int centerY, int externalRadius, int internalRadius, int angle, int width, int color);
int mgsDrawRotatedESector(int scrNumber, int centerX, int centerY, int radiusH, int radiusV, int startingAngle, int endingAngle, int rotationAngle, int width, int color);
int mgsDrawRotatedESegment(int scrNumber, int centerX, int centerY, int radiusH, int radiusV, int startingAngle, int endingAngle, int rotationAngle, int width, int color);
// Solid figures 
int mgsBorderedRectangle(int scrNumber, int left, int top, int right, int bottom, int color, int borderWidth, int borderColor);
int mgsBorderedEllipse(int scrNumber, int centerX, int centerY, int radiusH, int radiusV, int color, int borderWidth, int borderColor);
int mgsBorderedRegPolygonCR(int scrNumber, int sides, int centerX, int centerY, int radius, int color, int borderWidth, int borderColor);
int mgsBorderedRegPolygonVC(int scrNumber, int sides, int vertexX, int vertexY, int centerX, int centerY, int color, int borderWidth, int borderColor);
int mgsBorderedPolygon(int scrNumber, int sides, int *vertices, int color, int borderWidth, int borderColor);
int mgsBorderedStarCR(int scrNumber, int sides, int centerX, int centerY, int externalRadius, int internalRadius, int color, int borderWidth, int borderColor);
int mgsBorderedStarVC(int scrNumber, int sides, int vertexX, int vertexY, int centerX, int centerY, int internalRadius, int color, int borderWidth, int borderColor);
int mgsBorderedESector(int scrNumber, int centerX, int centerY, int radiusH, int radiusV, int startAngle, int endAngle, int color, int borderWidth, int borderColor);
int mgsBorderedESegment(int scrNumber, int centerX, int centerY, int radiusH, int radiusV, int startAngle, int endAngle, int color, int borderWidth, int borderColor);
// Rotated version
int mgsRotatedBorderedRectangle(int scrNumber, int left, int top, int right, int bottom, int angle, int color, int borderWidth, int borderColor);
int mgsRotatedBorderedEllipse(int scrNumber, int centerX, int centerY, int radiusH, int radiusV, int angle, int color, int borderWidth, int borderColor);
int mgsRotatedBorderedRegPolygonCR(int scrNumber, int sides, int centerX, int centerY, int radius, int angle, int color, int borderWidth, int borderColor);
int mgsRotatedBorderedPolygon(int scrNumber, int sides, int *vertices, int centerX, int centerY, int angle, int color, int borderWidth, int borderColor);
int mgsRotatedBorderedStarCR(int scrNumber, int sides, int centerX, int centerY, int externalRadius, int internalRadius, int angle, int color, int borderWidth, int borderColor);
int mgsRotatedBorderedESector(int scrNumber, int centerX, int centerY, int radiusH, int radiusV, int startAngle, int endAngle, int rotationAngle, int color, int borderWidth, int borderColor);
int mgsRotatedBorderedESegment(int scrNumber, int centerX, int centerY, int radiusH, int radiusV, int startAngle, int endAngle, int rotationAngle, int color, int borderWidth, int borderColor);
// Image
int mgsLoadImage(char *name);
int mgsSetImportTColor(unsigned int c);
int mgsMakeImage(int scrNumber, int originX, int originY, int width, int height);
int mgsImageWidth(int imgNumber);
int mgsImageHeight(int imgNumber);
int mgsDeleteImage(int imgNumber);
int mgsDisplayImage(int scrNumber, int imgNumber, int dx, int dy, int transparent);
int mgsDisplayRotatedImage(int scrNumber, int imgNumber, int dx, int dy, double scaleX, double scaleY, int angle, int transparent);
// Text output
int mgsSetFont(int scrNumber, int size, int textColor, int backColor);
int mgsSetFontType(int scrNumber, int size, int angle, char *fontName, int style);
int mgsSetFontColor(int scrNumber, int textColor, int backColor);
int mgsPuts(int scrNumber, int x, int y, char *str);
int mgsPutsAligned(int scrNumber, int mode, int left, int top, int right, int bottom, char *str);
int mgsTextSize(int scrNumber, char *str, int *y);
// Mouse input
int mgsMouseButtons(int scrNumber);
int mgsMouseX(int scrNumber);
int mgsMouseY(int scrNumber);
int mgsMouseGlobal(int *y);
int mgsMouseIn(int scrNumber);
int mgsMouseWheel(int scrNumber, unsigned char reset);
// Advanced mouse input
void mgsHandleMouse(int scr);
unsigned char mgsMouseClicked(int scr, unsigned char b, int x0, int y0, int x1, int y1);
unsigned char mgsMousePressed(int scr, unsigned char b, int x0, int y0, int x1, int y1);
unsigned char mgsMouseOver(int scr, int x0, int y0, int x1, int y1);
// Keyobard input
unsigned char mgsKeyPressed(int keyCode);
// Sound functions
int mgsPlaySoundFile(char *filename, int mode);
int mgsStopSound(void);
// Time and performance
void mgsSetFrameTime(int milliSeconds);
void mgsWaitFrame(void);
double mgsPerformanceMeasure(int mode);
// Error handling
void mgsShowError(int errNum, char *msg, int flags);
// System info
struct mgsInfo mgsVersionData(void);

// Macros
/**
 * \brief Creates a RGB color from its R, G and B components.
 *
 * This macro creates an int with the appropriate color by combining the R, G, and B
 * components passed individually. Each of them should be a value between 0 and 255
 */
#define mgsRGBColor(r,g,b) ((((b)&0xFF)<<16)|(((g)&0xFF)<<8)|((r)&0xFF))

/**
 * \brief Gets position and size information for an screen or the desktop.
 *
 * This macro calls the advanced get screen information but discarding information
 * about the handle
 */
#define mgsGetScreenInfo(s, p) mgsGetScreenInfoAdvanced((s), (p), NULL);

/** \name Macros to draw non bordered figures.
 * Non bordered versions of the functions to draw figures.
 */

/**@{ */
/** 
 * \brief Non bordered version of the corresponding function.
 */
#define mgsRectangle(s, l, t, r, b, c)  mgsBorderedRectangle((s), (l), (t), (r), (b), (c), 0, -1)
#define mgsEllipse(s, x, y, h, v, c)  mgsBorderedEllipse((s), (x), (y), (h), (v), (c), 0, -1)
#define mgsRegPolygonCR(s, n, x, y, r, c)  mgsBorderedRegPolygonCR((s), (n), (x), (y), (r), (c), 0, -1)
#define mgsRegPolygonVC(s, n, x, y, a, b, c)  mgsBorderedRegPolygonVC((s), (n), (x), (y), (a), (b), (c), 0, -1)
#define mgsPolygon(s, n, v, c)  mgsBorderedPolygon((s), (n), (v), (c), 0, -1)
#define mgsStarCR(s, n, x, y, e, i, c)  mgsBorderedStarCR((s), (n), (x), (y), (e), (i), (c), 0, -1)
#define mgsStarVC(s, n, x, y, a, b, i, c)  mgsBorderedStarVC((s), (n), (x), (y), (a), (b), (i), (c), 0, -1)
#define mgsESector(s, x, y, h, v, i, e, c) mgsBorderedESector((s), (x), (y), (h), (v), (i), (e), (c), 0, -1)
#define mgsESegment(s, x, y, h, v, i, e, c) mgsBorderedESegment((s), (x), (y), (h), (v), (i), (e), (c), 0, -1)
#define mgsRotatedRectangle(s, l, t, r, b, a, c)  mgsRotatedBorderedRectangle((s), (l), (t), (r), (b), (a), (c), 0, -1)
#define mgsRotatedEllipse(s, x, y, h, v, a, c)  mgsRotatedBorderedEllipse((s), (x), (y), (h), (v), (a), (c), 0, -1)
#define mgsRotatedRegPolygonCR(s, n, x, y, r, a, c)  mgsRotatedBorderedRegPolygonCR((s), (n), (x), (y), (r), (a), (c), 0, -1)
#define mgsRotatedPolygon(s, n, v, x, y, a, c)  mgsRotatedBorderedPolygon((s), (n), (v), (x), (y), (a), (c), 0, -1)
#define mgsRotatedStarCR(s, n, x, y, e, i, a, c)  mgsRotatedBorderedStarCR((s), (n), (x), (y), (e), (i), (a), (c), 0, -1)
#define mgsRotatedESector(s, x, y, h, v, i, e, a, c) mgsRotatedBorderedESector((s), (x), (y), (h), (v), (i), (e), (a), (c), 0, -1)
#define mgsRotatedESegment(s, x, y, h, v, i, e, a, c) mgsRotatedBorderedESegment((s), (x), (y), (h), (v), (i), (e), (a), (c), 0, -1)
/**@} */

// Button identifiers for the macros below
#define _MLEFT		0	
#define _MCENTER	1
#define _MRIGHT		2

/**@{ */
/** 
 * \brief Individual macros to select only one mouse button on the advanced mouse functions.
 */
#define mgsLeftMouseClk(s, x, y, a, b)		mgsMouseClicked(s, _MLEFT, x, y, a, b)
#define mgsCenterMouseClk(s, x, y, a, b)	mgsMouseClicked(s, _MCENTER, x, y, a, b)
#define mgsRightMouseClk(s, x, y, a, b)		mgsMouseClicked(s, _MRIGHT, x, y, a, b)

#define mgsLeftMousePress(s, x, y, a, b)	mgsMousePressed(s, _MLEFT, x, y, a, b)
#define mgsCenterMousePress(s, x, y, a, b)	mgsMousePressed(s, _MCENTER, x, y, a, b)
#define mgsRightMousePress(s, x, y, a, b)	mgsMousePressed(s, _MRIGHT, x, y, a, b)
/**@} */

// Buttons
#define MGS_MOUSELEFT		0x01
#define MGS_MOUSERIGHT		0x02
#define MGS_MOUSECENTER		0x04
// Constantes para indicar como se ha pulsado un botn del raton al
// respecto de la posicin del cursor sobre un rectngulo. 
#define MGS_MNO		0x00	// No pulsado
#define MGS_MOUT	0x01	// Pulsado fuera
#define MGS_MIN		0x02	// Pulsado dentro
#define MGS_MNEW	0x10	// Se ha detectado pulsado por primera vez

// Desktop info for mgsGetInfo
#define MGS_DESKTOP	-1

// Performance measurements
#define MGS_PMRESET			0x01
#define MGS_PMUSED			0x02
#define MGS_PMRELATIVE		0x04

// Error codes
#define MGS_ERRSYSINIT		-100  // System. We cannot initialize
#define MGS_ERRNOSCRLEFT	-1101 // User. All the screens are used
#define MGS_ERRCRLOOP		-102  // System. Cannot create thread
#define MGS_ERRSCRNOEXIST	-1103 // User. The screen number used does not exist
#define MGS_ERRGROBJ		-104  // System. Error creating graphic object
#define MGS_ERRNOIMGLEFT	-1105 // User. All the images are used
#define MGS_ERRIMGNOEXIST	-1106 // User. The image number used does not exist
#define MGS_ERRINCDATA		-1107 // User. Cannot work with the given data
#define MGS_ERRSYSTEM		-108  // System. Generic system error
#define MGS_ERRNOINIT		-1109 // User. No screen has been created
#define MGS_ERRNORESLEFT	-1110 // User. All the resources of the requested type are used
#define MGS_ERRRESNOEXIST	-1111 // User. The resource number used does not exist

// Error flags
#define MGS_EFWAIT	0x01
#define MGS_EFWIN	0x02

// Colors
#define MGS_CWHITE	0x00FFFFFF
#define MGS_CGREY	0x007F7F7F
#define MGS_CBLACK	0x00000000
#define MGS_CRED	0x000000FF
#define MGS_CGREEN	0x0000FF00
#define MGS_CBLUE	0x00FF0000
#define MGS_CCYAN	0x00FFFF00
#define MGS_CMAGENTA 0x00FF00FF
#define MGS_CYELLOW	0x0000FFFF

// Text
#define MGS_TRANSPARENT	0x80000000
#define MGS_ITALIC 0x00000001
#define MGS_UNDERLINE 0x00000002
#define MGS_STRIKEOUT 0x00000004
#define MGS_LIGHT (300 << 3)
#define MGS_BOLD (700 << 3)
#define MGS_HEAVY (900 << 3)
#define MGS_TLEFT		0
#define MGS_TRIGHT		1
#define MGS_TTOP		0
#define MGS_TBOTTOM		2
#define MGS_THCENTER	4
#define MGS_TVCENTER	8
#define MGS_TCUT 		16
#define MGS_TSCREEN		32


// Keys
#define MGS_KBACK		0x08
#define MGS_KTAB		0x09
#define MGS_KCLEAR		0x0C
#define MGS_KRETURN		0x0D
#define MGS_KSHIFT		0x10
#define MGS_KCONTROL	0x11
#define MGS_KALT		0x12
#define MGS_KPAUSE		0x13
#define MGS_KCAPSLK		0x14
#define MGS_KESC		0x1B
#define MGS_KSPACE		0x20
#define MGS_KPGUP		0x21
#define MGS_KPGDOWN		0x22
#define MGS_KEND		0x23
#define MGS_KHOME		0x24
#define MGS_KLEFTARROW	0x25
#define MGS_KUPARROW	0x26
#define MGS_KRIGHTARROW	0x27
#define MGS_KDOWNARROW	0x28
#define MGS_KSELECT		0x29
#define MGS_KPRINT		0x2A
#define MGS_KEXECUTE	0x2B
#define MGS_KSNAPSHOT	0x2C
#define MGS_KINSERT		0x2D
#define MGS_KDELETE		0x2E
#define MGS_KHELP		0x2F
#define MGS_KLEFTWIN	0x5B
#define MGS_KRIGHTWIN	0x5C
#define MGS_KAPPS		0x5D
#define MGS_KSLEEP		0x5F
#define MGS_KNUMPAD0	0x60
#define MGS_KNUMPAD1	0x61
#define MGS_KNUMPAD2	0x62
#define MGS_KNUMPAD3	0x63
#define MGS_KNUMPAD4	0x64
#define MGS_KNUMPAD5	0x65
#define MGS_KNUMPAD6	0x66
#define MGS_KNUMPAD7	0x67
#define MGS_KNUMPAD8	0x68
#define MGS_KNUMPAD9	0x69
#define MGS_KMULTIPLY	0x6A
#define MGS_KADD		0x6B
#define MGS_KSEPARATOR	0x6C
#define MGS_KSUBTRACT	0x6D
#define MGS_KDECIMAL	0x6E
#define MGS_KDIVIDE		0x6F
#define MGS_KF1			0x70
#define MGS_KF2			0x71
#define MGS_KF3			0x72
#define MGS_KF4			0x73
#define MGS_KF5			0x74
#define MGS_KF6			0x75
#define MGS_KF7			0x76
#define MGS_KF8			0x77
#define MGS_KF9			0x78
#define MGS_KF10		0x79
#define MGS_KF11		0x7A
#define MGS_KF12		0x7B
#define MGS_KNUMLOCK	0x90
#define MGS_KSCROLL		0x91
#define MGS_KLSHIFT		0xA0
#define MGS_KRSHIFT		0xA1
#define MGS_KLCONTROL	0xA2
#define MGS_KRCONTROL	0xA3
#define MGS_KLMENU		0xA4
#define MGS_KRMENU		0xA5

// Sound
#define MGS_LOOP		1

#endif
