/**
 * \file mrmWidgets.h
 * \author Germ&aacute;n Fabregat
 * \date October 2017
 * \brief Header file for the MGS Library Widgets
 */

#ifndef _MRMWIDGETS_H
#define _MRMWIDGETS_H

// Public data types
/*
 * Structure to retrieve information about the mgs library
 * using the mgsVersionData() function.
 */
struct mrmWGInfo{
	int vMajor; // Major version number.
	int vMinor; // Minor version number.
	int	maxSlider; // Maximum number of sliders.
	int freeSlider; // Current number of free sliders to be used.
    int maxButton; // Maximum number of buttons.
	int freeButton; // Current number of free buttons to be used.
	int	maxTextBox; // Maximum number of text boxes.
	int freeTextBox; // Current number of free text boxes to be used.
    int maxSelection; // Maximum number of selections.
	int freeSelection; // Current number of free selections to be used.
	int maxList; // Maximum number of lists.
	int freeList; // Current number of free lists to be used.
};

/**@{ */
/** 
 * \brief 3D Rectangle construction constants.
 */
// 3D rectangle
#define MWG_3DROUT		0x10	/**Projected out*/
#define MWG_3DRIN		0		/**Projected in*/
#define MWG_3DRNORMAL	2		/**Normal projection*/
#define MWG_3DRLOW		1		/**Low projection*/
/**@} */

// Keyboard types for textboxes
#define MWG_KBNUMERIC	1		// NumPad
#define MWG_KBALPHA		2		// Keyboard

// Direction for Sliders
#define MWG_SLHORIZONTAL	1
#define MWG_SLVERTICAL		2

// Color Picker. Default colors
#define MWG_CPBCKCOL 	0x808080
#define MWG_CPTXTCOL	MGS_CWHITE

// Selection. Mode
#define MWG_SCUNIQUE	0x0000
#define MWG_SCMULTI		0x1000

#define MWG_SCROUND		0x000
#define MWG_SCSQUARE	0x100
#define MWG_SCSYMBOL	0x200
#define MWG_SCNONE		0x400

// Lists. Mode
#define MWG_LINONE		0
#define MWG_LIORDERED	1

// Lists. Callback instant
#define MWG_LIOPEN		0
#define MWG_LICLOSE     1
#define MWG_LISELECT	2

// Utilities
void mrmWU3DRectangle(int scr, int x0, int y0, int x1, int y1, int col, int type);
// System info
struct mrmWGInfo mrmWVersionData(void);
// Working functions
void mrmWShow(void);
void mrmWLoop(void);

// NumPad
int mrmNPInit(int bgCol, int nChar, void (*callBack)(void));
void mrmNPDelete(void);
int mrmNPExists(void);
void mrmNPMove(int x, int y);
void mrmNPSetColors(int colKeyNum, int colKeyEsp, int colScr, int colKeyTxt, int colScrTxt, int colDisTxt);
void mrmNPGetText(char *text, int size, int *blocked);
double mrmNPGetVal(int *blocked);

// Keyboard
int mrmKeyboardInit(int bgCol, int nChar, void (*callBack)(void));
void mrmKeyboardDelete(void);
int mrmKeyboardExists(void);
void mrmKeyboardMove(int x, int y);
void mrmKeyboardSetColors(int colKeyChar, int colKeyEsp, int colAltOff, int colAltOn, int colScr, int colKeyTxt, int colScrTxt, int colDisTxt);
void mrmKeyboardGetText(char *res, int *blocked);

// Color Picker
int mrmCPInit(int bgCol, int txtCol);
void mrmCPDelete(void);
int mrmCPExists(void);
void mrmCPMove(int x, int y);
void mrmCPSetSelection(int crgb);
int mrmCPRGB(void);

// Button
int mrmButtonInit(int scr, int width, int height, void (*callBack)(int, int), char *label, int nCol, int pCol, int dCol);
int mrmButtonDelete(int idx);
int mrmButtonSetEnabled(int idx, int enabled);
int mrmButtonSetPos(int idx, int x, int y);
int mrmButtonSetSize(int idx, int w, int h);
int mrmButtonSetColor(int idx, int nCol, int pCol, int dCol);
int mrmButtonSetFont(int idx, int sz, int a, char *fn, int s);
int mrmButtonSetFontColor(int idx, int nCol, int pCol, int dCol);
int mrmButtonSetLabel(int idx, char *label);
int mrmButtonSetCounter(int idx, unsigned int val);
int mrmButtonGetCounter(int idx);

#define mrmButtonDisable(x)	mrmButtonSetEnabled((x), 0)
#define mrmButtonEnable(x)	mrmButtonSetEnabled((x), 1)

// Text Box
int mrmTextBoxInit(int scr, int width, int height, int nChar, int kbType, void (*callBack)(int, int), int col, int dCol);
int mrmTextBoxDelete(int idx);
int mrmTextBoxSetEnabled(int idx, int enabled);
int mrmTextBoxSetPos(int idx, int x, int y);
int mrmTextBoxSetSize(int idx, int w, int h);
int mrmTextBoxSetColor(int idx, int col, int dCol);
int mrmTextBoxSetFont(int idx, int sz, int a, char *fn, int s);
int mrmTextBoxSetFontColor(int idx, int nCol, int dCol);
int mrmTextBoxSetText(int idx, char *string);
int mrmTextBoxGetText(int idx, char *string);

#define mrmTextBoxDisable(x)	mrmTextBoxSetEnabled((x), 0)
#define mrmTextBoxEnable(x)		mrmTextBoxSetEnabled((x), 1)

// Slider
int mrmSliderInit(int scr, int orientation, int width, int height, double min, double max, int col, int cCol, int cBcol, int dCol);
int mrmSliderDelete(int idx);
int mrmSliderSetEnabled(int idx, int enabled);
int mrmSliderSetPos(int idx, int x, int y);
int mrmSliderSetColor(int idx, int col, int cCol, int cBcol, int dCol);
double mrmSliderSetVal(int idx, double val);
double mrmSliderGetVal(int idx);

#define mrmSliderDisable(x)	mrmSliderSetEnabled((x), 0)
#define mrmSliderEnable(x)	mrmSliderSetEnabled((x), 1)

// Selection
int mrmSelectionInit(int scr, int width, int height, int nSel, char **text, int cols, int mode);
int mrmSelectionDelete(int idx);
int mrmSelectionSetEnabled(int idx, int enabled);
int mrmSelectionSetPos(int idx, int x, int y);
int mrmSelectionSetOptionsPos(int idx, int *points);
int mrmSelectionSetColor(int idx, int col, int cCol, int cBcol, int dCol);
int mrmSelectionSetFont(int idx, int sz, int a, char *fn, int s);
int mrmSelectionSetFontColor(int idx, int nCol, int sCol, int dCol, int dsCol);
int mrmSelectionSetVal(int idx, int val);
int mrmSelectionGetVal(int idx);

#define mrmSelectionDisable(x)	mrmSelectionSetEnabled((x), 0)
#define mrmSelectionEnable(x)	mrmSelectionSetEnabled((x), 1)

// List
int mrmListInit(int scr, int width, int height, int wwidth, int mode, char **texts, void (*callBack)(int, int, int), int nSel, char *title);
int mrmListDelete(int idx);
int mrmListSetEnabled(int idx, int enabled);
int mrmListSetPos(int idx, int x, int y);
int mrmListSetSize(int idx, int w, int h, int ww);
int mrmListSetColor(int idx, int col, int dCol, int sCol, int wCol, int wBCol, int wW1Col, int wW2Col);
int mrmListSetFont(int idx, int sz, int a, char *fn, int s);
int mrmListSetFontColor(int idx, int nCol, int dCol, int tsCol);
int mrmListAddItems(int idx, char **items);
int mrmListInsertItem(int idx, int pos, char *item);
int mrmListDelItem(int idx, int pos);
int mrmListSetSelected(int idx, int sel);
int mrmListGetSelected(int idx);
int mrmListItemAtIndex(int idx, int pos, char *string);

#define mrmListDisable(x)	mrmListSetEnabled((x), 0)
#define mrmListEnable(x)	mrmListSetEnabled((x), 1)

#endif
